/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.data;
/*uri是统一资源标志符（Uniform Resource Identifier），而在这个类中定义了各种Uri常量，
以便在其他的类中调用。Uri用处很多，涉及到安卓权限，比如通过Uri可以：
直接拨打电话、调出拨打电话界面、调WEB浏览器、卸载、安装、播放、调用发邮件、直接发邮件等系统功能
Android上可用的每种资源 -包括 图像、视频片段、音频资源等都可以用Uri来表示*/
import android.net.Uri;
public class Notes {
    /*便签数据库,该类主要实现了对便签的相关属性包括：Anthority, Tag，以及数据，联系人的信息进行保存管理，
    其中主要实现了对若干属性的定义，并创建了DataColumns，DataColumns接口的定义，该两个接口可用于其余类的实现，
    在便签管理中实现TextNote,CallNote两个类，用于对便签内容的保存。*/
    public static final String AUTHORITY = "micode_notes";//定义了一个权限名，这里是为了符合Uri的格式而定义的。
    public static final String TAG = "Notes";//定义该类的名称
    public static final int TYPE_NOTE     = 0;//Note的类型，这里规定了三种；普通Note，
    public static final int TYPE_FOLDER   = 1;//文件夹Folder，
    public static final int TYPE_SYSTEM   = 2;//系统文件System

    /**
     * Following IDs are system folders' identifiers（下列ID是系统文件的标识）
     * {@link Notes#ID_ROOT_FOLDER } is default folder（根目录（默认文件夹））
     * {@link Notes#ID_TEMPARAY_FOLDER } is for notes belonging no folder（临时文件夹）
     * {@link Notes#ID_CALL_RECORD_FOLDER} is to store call records（通话记录文件夹）
     * {@link Notes#ID_TRASH_FOLER}（垃圾文件夹）
     */
    public static final int ID_ROOT_FOLDER = 0;
    public static final int ID_TEMPARAY_FOLDER = -1;
    public static final int ID_CALL_RECORD_FOLDER = -2;
    public static final int ID_TRASH_FOLER = -3;
    //下列是各个文件夹名称标识，定义各种小部件（widget）
    public static final String INTENT_EXTRA_ALERT_DATE = "net.micode.notes.alert_date";
    public static final String INTENT_EXTRA_BACKGROUND_ID = "net.micode.notes.background_color_id";
    public static final String INTENT_EXTRA_WIDGET_ID = "net.micode.notes.widget_id";
    public static final String INTENT_EXTRA_WIDGET_TYPE = "net.micode.notes.widget_type";
    public static final String INTENT_EXTRA_FOLDER_ID = "net.micode.notes.folder_id";
    public static final String INTENT_EXTRA_CALL_DATE = "net.micode.notes.call_date";

    public static final int TYPE_WIDGET_INVALIDE      = -1;//定义查询便签和文件夹的指针
    public static final int TYPE_WIDGET_2X            = 0;//2*2小部件
    public static final int TYPE_WIDGET_4X            = 1;//4*4小部件

    public static class DataConstants {
        //存放textnotes和callnotes地址
        public static final String NOTE = TextNote.CONTENT_ITEM_TYPE;
        public static final String CALL_NOTE = CallNote.CONTENT_ITEM_TYPE;
    }

    /**
     * Uri to query all notes and folders（统一资源标识符查询便签和文件夹）
     */
    public static final Uri CONTENT_NOTE_URI = Uri.parse("content://" + AUTHORITY + "/note");

    /**
     * Uri to query data（统一资源标识符查询数据）
     */
    public static final Uri CONTENT_DATA_URI = Uri.parse("content://" + AUTHORITY + "/data");

    public interface NoteColumns {
        //定义NoteColumns的常量,用于后面创建数据库的表头，定义Note的一系列属性
        /**
         * The unique ID for a row（主键）
         * <P> Type: INTEGER (long) </P>
         */
        public static final String ID = "_id";

        /**
         * The parent's id for note or folder
         * <P> Type: INTEGER (long) </P>
         */
        public static final String PARENT_ID = "parent_id";//父节点id

        /**
         * Created data for note or folder
         * <P> Type: INTEGER (long) </P>
         */
        public static final String CREATED_DATE = "created_date";

        /**
         * Latest modified date
         * <P> Type: INTEGER (long) </P>
         */
        public static final String MODIFIED_DATE = "modified_date";//更新日期


        /**
         * Alert date
         * <P> Type: INTEGER (long) </P>
         */
        public static final String ALERTED_DATE = "alert_date";//更改日期

        /**
         * Folder's name or text content of note
         * <P> Type: TEXT </P>
         */
        public static final String SNIPPET = "snippet";

        /**
         * Note's widget id
         * <P> Type: INTEGER (long) </P>
         */
        public static final String WIDGET_ID = "widget_id";

        /**
         * Note's widget type
         * <P> Type: INTEGER (long) </P>
         */
        public static final String WIDGET_TYPE = "widget_type";

        /**
         * Note's background color's id
         * <P> Type: INTEGER (long) </P>
         */
        public static final String BG_COLOR_ID = "bg_color_id";

        /**
         * For text note, it doesn't has attachment, for multi-media
         * note, it has at least one attachment
         * <P> Type: INTEGER </P>
         */
        public static final String HAS_ATTACHMENT = "has_attachment";//是否有附件

        /**
         * Folder's count of notes
         * <P> Type: INTEGER (long) </P>
         */
        public static final String NOTES_COUNT = "notes_count";

        /**
         * The file type: folder or note
         * <P> Type: INTEGER </P>
         */
        public static final String TYPE = "type";

        /**
         * The last sync id
         * <P> Type: INTEGER (long) </P>
         */
        public static final String SYNC_ID = "sync_id";//最后一次的同步id

        /**
         * Sign to indicate（显示） local modified or not（本地是否修改过标识）
         * <P> Type: INTEGER </P>
         */
        public static final String LOCAL_MODIFIED = "local_modified";//本地是否修改过

        /**
         * Original parent id before moving into temporary folder
         * <P> Type : INTEGER </P>
         */
        public static final String ORIGIN_PARENT_ID = "origin_parent_id";//父节点id在移到临时文件夹之前

        /**
         * The gtask id
         * <P> Type : TEXT </P>
         */
        public static final String GTASK_ID = "gtask_id";//后台任务的id

        /**
         * The version code
         * <P> Type : INTEGER (long) </P>
         */
        public static final String VERSION = "version";//版本号
    }

    public interface DataColumns {
        //定义DataColumns的常量,用于后面创建数据库的表头。主要是定义存储便签数据内容
        /**
         * The unique ID for a row
         * <P> Type: INTEGER (long) </P>
         */
        public static final String ID = "_id";//每一行id

        /**
         * The MIME type of the item represented by this row.（这行代表的mime类型）
         * <P> Type: Text </P>
         */
        public static final String MIME_TYPE = "mime_type";//MIME类型能包含视频、图像、文本、音频、应用程序等数据

        /**
         * The reference id to note that this data belongs to
         * <P> Type: INTEGER (long) </P>
         */
        public static final String NOTE_ID = "note_id";//标签id

        /**
         * Created data for note or folder
         * <P> Type: INTEGER (long) </P>
         */
        public static final String CREATED_DATE = "created_date";//创建时间

        /**
         * Latest modified date
         * <P> Type: INTEGER (long) </P>
         */
        public static final String MODIFIED_DATE = "modified_date";//修改时间

        /**
         * Data's content
         * <P> Type: TEXT </P>
         */
        public static final String CONTENT = "content";//内容

        /*以下定义了五个常量*/
        /**
         * Generic data column, the meaning is {@link #MIMETYPE} specific, used for
         * integer data type
         * <P> Type: INTEGER </P>
         */
        public static final String DATA1 = "data1";
        /**
         * Generic data column, the meaning is {@link #MIMETYPE} specific, used for
         * integer data type
         * <P> Type: INTEGER </P>
         */
        public static final String DATA2 = "data2";

        /**
         * Generic data column, the meaning is {@link #MIMETYPE} specific, used for
         * TEXT data type
         * <P> Type: TEXT </P>
         */
        public static final String DATA3 = "data3";

        /**
         * Generic data column, the meaning is {@link #MIMETYPE} specific, used for
         * TEXT data type
         * <P> Type: TEXT </P>
         */
        public static final String DATA4 = "data4";

        /**
         * Generic data column, the meaning is {@link #MIMETYPE} specific, used for
         * TEXT data type
         * <P> Type: TEXT </P>
         */
        public static final String DATA5 = "data5";
    }

    public static final class TextNote implements DataColumns {
        //文本标签实现DataColumns接口，继承上述所有常量
        /*模式指示文本是否处于检查列表模式
        1：检查列表模式，0：正常模式*/
        /**
         * Mode to indicate the text in check list mode or not
         * <P> Type: Integer 1:check list mode 0: normal mode </P>
         */
        public static final String MODE = DATA1;//继承DATA1

        public static final int MODE_CHECK_LIST = 1;//1：检查列表模式

        public static final String CONTENT_TYPE = "vnd.android.cursor.dir/text_note";//内容类型

        public static final String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/text_note";//内容项目的类型

        public static final Uri CONTENT_URI = Uri.parse("content://" + AUTHORITY + "/text_note");//content 的索引标识符
    }

    public static final class CallNote implements DataColumns {
        //记录通话数据的表头
        /**
         * Call date for this record
         * <P> Type: INTEGER (long) </P>
         */
        public static final String CALL_DATE = DATA1;//通话日期

        /**
         * Phone number for this record
         * <P> Type: TEXT </P>
         */
        public static final String PHONE_NUMBER = DATA3;//电话号码

        public static final String CONTENT_TYPE = "vnd.android.cursor.dir/call_note";//内容类型

        public static final String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/call_note";//内容项目类型

        public static final Uri CONTENT_URI = Uri.parse("content://" + AUTHORITY + "/call_note");//content 的索引标识符
    }
}
